@@ -7,14 +7,14 @@ use dataplane::core::Encoder;
 use dataplane::core::Decoder;
 use fluvio_sc_schema::objects::{Metadata, AllCreatableSpec};
 use fluvio_sc_schema::AdminRequest;
-use fluvio_socket::FlvSocketError;
+use fluvio_socket::SocketError;
 use fluvio_socket::MultiplexerSocket;
-//use fluvio_future::native_tls::AllDomainConnector;
 
 use crate::sockets::{ClientConfig, VersionedSerialSocket, SerialFrame};
 use crate::{FluvioError, FluvioConfig};
 use crate::metadata::objects::{ListResponse, ListSpec, DeleteSpec, CreateRequest};
 use crate::config::ConfigFile;
+use crate::sync::MetadataStores;
 
 /// An interface for managing a Fluvio cluster
 ///
@@ -48,11 +48,15 @@ use crate::config::ConfigFile;
 /// [`Fluvio`]: ./struct.Fluvio.html
 /// [`connect`]: ./struct.FluvioAdmin.html#method.connect
 /// [`connect_with_config`]: ./struct.FluvioAdmin.html#method.connect_with_config
-pub struct FluvioAdmin(VersionedSerialSocket);
+pub struct FluvioAdmin {
+    socket: VersionedSerialSocket,
+    #[allow(dead_code)]
+    metadata: MetadataStores,
+}
 
 impl FluvioAdmin {
-    pub(crate) fn new(client: VersionedSerialSocket) -> Self {
-        Self(client)
+    pub(crate) fn new(socket: VersionedSerialSocket, metadata: MetadataStores) -> Self {
+        Self { socket, metadata }
     }
 
     /// Creates a new admin connection using the current profile from `~/.fluvio/config`
@@ -111,27 +115,26 @@ impl FluvioAdmin {
 
         let (socket, config, versions) = inner_client.split();
         let socket = MultiplexerSocket::shared(socket);
-
+        let metadata = MetadataStores::start(socket.clone()).await?;
         let versioned_socket = VersionedSerialSocket::new(socket, config, versions);
-        Ok(Self(versioned_socket))
+
+        Ok(Self {
+            socket: versioned_socket,
+            metadata,
+        })
     }
 
     #[instrument(skip(self, request))]
-    async fn send_receive<R>(&mut self, request: R) -> Result<R::Response, FlvSocketError>
+    async fn send_receive<R>(&self, request: R) -> Result<R::Response, SocketError>
     where
         R: AdminRequest + Send + Sync,
     {
-        self.0.send_receive(request).await
+        self.socket.send_receive(request).await
     }
 
     /// create new object
     #[instrument(skip(self, name, dry_run, spec))]
-    pub async fn create<S>(
-        &mut self,
-        name: String,
-        dry_run: bool,
-        spec: S,
-    ) -> Result<(), FluvioError>
+    pub async fn create<S>(&self, name: String, dry_run: bool, spec: S) -> Result<(), FluvioError>
     where
         S: Into<AllCreatableSpec>,
     {
@@ -149,7 +152,7 @@ impl FluvioAdmin {
     /// delete object by key
     /// key is depend on spec, most are string but some allow multiple types
     #[instrument(skip(self, key))]
-    pub async fn delete<S, K>(&mut self, key: K) -> Result<(), FluvioError>
+    pub async fn delete<S, K>(&self, key: K) -> Result<(), FluvioError>
     where
         S: DeleteSpec,
         K: Into<S::DeleteKey>,
@@ -160,7 +163,7 @@ impl FluvioAdmin {
     }
 
     #[instrument(skip(self, filters))]
-    pub async fn list<S, F>(&mut self, filters: F) -> Result<Vec<Metadata<S>>, FluvioError>
+    pub async fn list<S, F>(&self, filters: F) -> Result<Vec<Metadata<S>>, FluvioError>
     where
         S: ListSpec + Encoder + Decoder,
         S::Status: Encoder + Decoder,
@@ -179,95 +182,36 @@ impl FluvioAdmin {
             .try_into()
             .map_err(|err| Error::new(ErrorKind::Other, format!("can't convert: {}", err)).into())
     }
-
-    /*
-    /// Connect to replica leader for a topic/partition
-    async fn find_replica_for_topic_partition(
-        &mut self,
-        topic: &str,
-        partition: i32,
-    ) -> Result<Self::Leader, ClientError> {
-        debug!(
-            "trying to find replica for topic: {}, partition: {}",
-            topic, partition
-        );
-
-        let topic_comp_resp = self.get_topic_composition(topic).await?;
-
-        trace!("topic composition: {:#?}", topic_comp_resp);
-
-        let mut topics_resp = topic_comp_resp.topics;
-        let spus_resp = topic_comp_resp.spus;
-
-        // there must be one topic in reply
-        if topics_resp.len() != 1 {
-            return Err(ClientError::IoError(IoError::new(
-                ErrorKind::InvalidData,
-                format!("topic error: expected 1 topic, found {}", topics_resp.len()),
-            )));
+}
+
+#[cfg(feature = "unstable")]
+mod unstable {
+    use super::*;
+    use futures_util::Stream;
+    use crate::sync::AlwaysNewContext;
+    use crate::metadata::topic::TopicSpec;
+    use crate::metadata::partition::PartitionSpec;
+    use crate::metadata::spu::SpuSpec;
+    use crate::metadata::store::MetadataChanges;
+
+    impl FluvioAdmin {
+        /// Create a stream that yields updates to Topic metadata
+        pub fn watch_topics(
+            &self,
+        ) -> impl Stream<Item = MetadataChanges<TopicSpec, AlwaysNewContext>> {
+            self.metadata.topics().watch()
         }
 
-        let topic_resp = topics_resp.remove(0);
-
-        if topic_resp.error_code != FlvErrorCode::None {
-            if topic_resp.error_code == FlvErrorCode::TopicNotFound {
-                return Err(ClientError::TopicNotFound(topic.to_owned()));
-            } else {
-                return Err(ClientError::IoError(IoError::new(
-                    ErrorKind::InvalidData,
-                    format!(
-                        "error during topic lookup: {}",
-                        topic_resp.error_code.to_sentence()
-                    ),
-                )));
-            }
+        /// Create a stream that yields updates to Partition metadata
+        pub fn watch_partitions(
+            &self,
+        ) -> impl Stream<Item = MetadataChanges<PartitionSpec, AlwaysNewContext>> {
+            self.metadata.partitions().watch()
         }
-        // lookup leader
-        for partition_resp in topic_resp.partitions {
-            if partition_resp.partition_idx == partition {
-                // check for errors
-                if partition_resp.error_code != FlvErrorCode::None {
-                    return Err(ClientError::IoError(IoError::new(
-                        ErrorKind::InvalidData,
-                        format!(
-                            "topic-composition partition error: {}",
-                            topic_resp.error_code.to_sentence()
-                        ),
-                    )));
-                }
-
-                // traverse spus and find leader
-                let leader_id = partition_resp.leader_id;
-                for spu_resp in &spus_resp {
-                    if spu_resp.spu_id == leader_id {
-                        // check for errors
-                        if spu_resp.error_code != FlvErrorCode::None {
-                            return Err(ClientError::IoError(IoError::new(
-                                ErrorKind::InvalidData,
-                                format!(
-                                    "problem with partition look up {}:{} error: {}",
-                                    topic,
-                                    partition,
-                                    topic_resp.error_code.to_sentence()
-                                ),
-                            )));
-                        }
-
-                        debug!("spu {}/{}: is leader", spu_resp.host, spu_resp.port);
 
-                        let mut leader_client_config = self.0.config().clone();
-                        let addr: ServerAddress = spu_resp.into();
-                        leader_client_config.set_addr(addr.to_string());
-
-                        let client = leader_client_config.connect().await?;
-                        let leader_config = ReplicaLeaderConfig::new(topic.to_owned(), partition);
-                        return Ok(SpuReplicaLeader::new(leader_config, client));
-                    }
-                }
-            }
+        /// Create a stream that yields updates to SPU metadata
+        pub fn watch_spus(&self) -> impl Stream<Item = MetadataChanges<SpuSpec, AlwaysNewContext>> {
+            self.metadata.spus().watch()
         }
-
-        Err(ClientError::PartitionNotFound(topic.to_owned(), partition))
     }
-    */
 }
